@@ -4,7 +4,7 @@ from .middleware import CacheMiddleware
 
 
 def cache_page(*args, **kwargs):
-    return (
-        decorator_from_middleware_with_args
-        (CacheMiddleware)(*args, **kwargs)
+    cache_alias = kwargs.pop("cache", None)
+    return decorator_from_middleware_with_args(CacheMiddleware)(
+        cache_alias=cache_alias, *args, **kwargs
     )
