@@ -1,23 +1,27 @@
-import cgi
 import functools
-import urllib
+import logging
 
 from django.core.exceptions import ImproperlyConfigured
 from django.conf import settings
-from django.core.cache import cache
+from django.core.cache import caches, DEFAULT_CACHE_ALIAS
 from django.utils.encoding import iri_to_uri
 from django.utils.cache import (
     get_cache_key,
     learn_cache_key,
     patch_response_headers,
-    get_max_age
+    get_max_age,
 )
+from urllib.parse import parse_qs, urlencode
 
 from fancy_cache.utils import md5
 
+LOGGER = logging.getLogger(__name__)
 
-REMEMBERED_URLS_KEY = 'fancy-urls'
+REMEMBERED_URLS_KEY = "fancy-urls"
 LONG_TIME = 60 * 60 * 24 * 30
+USE_MEMCACHED_CAS = getattr(
+    settings, "FANCY_USE_MEMCACHED_CHECK_AND_SET", False
+)
 
 
 class RequestPath(object):
@@ -32,19 +36,13 @@ class RequestPath(object):
         if self.only_get_keys is not None:
             # then monkey patch self.request.get_full_path
             self.request.get_full_path = functools.partial(
-                self.get_full_path,
-                self.request,
-                self.only_get_keys,
-                True
+                self.get_full_path, self.request, self.only_get_keys, True
             )
 
         if self.forget_get_keys is not None:
             # then monkey patch self.request.get_full_path
             self.request.get_full_path = functools.partial(
-                self.get_full_path,
-                self.request,
-                self.forget_get_keys,
-                False
+                self.get_full_path, self.request, self.forget_get_keys, False
             )
 
     def __exit__(self, exc_type, exc_value, traceback):
@@ -56,14 +54,14 @@ class RequestPath(object):
         with the ability to return a different query string based on
         `keys` to be included or excluded
         """
-        qs = this.META.get('QUERY_STRING', '')
-        parsed = cgi.parse_qs(qs)
-        if is_only_keys :
+        qs = this.META.get("QUERY_STRING", "")
+        parsed = parse_qs(qs, keep_blank_values=True)
+        if is_only_keys:
             keep = dict((k, parsed[k]) for k in parsed if k in keys)
-        else :
+        else:
             keep = dict((k, parsed[k]) for k in parsed if k not in keys)
-        qs = urllib.urlencode(keep, True)
-        return '%s%s' % (this.path, ('?' + iri_to_uri(qs)) if qs else '')
+        qs = urlencode(keep, True)
+        return "%s%s" % (this.path, ("?" + iri_to_uri(qs)) if qs else "")
 
 
 class UpdateCacheMiddleware(object):
@@ -79,12 +77,12 @@ class UpdateCacheMiddleware(object):
     def process_response(self, request, response):
         """Sets the cache, if needed."""
         if (
-            not hasattr(request, '_cache_update_cache') or
-            not request._cache_update_cache
+            not hasattr(request, "_cache_update_cache")
+            or not request._cache_update_cache
         ):
             # We don't need to update the cache, just return.
             return response
-        if request.method != 'GET':
+        if request.method != "GET":
             # This is a stronger requirement than above. It is needed
             # because of interactions between this middleware and the
             # HTTPMiddleware, which throws the body of a HEAD-request
@@ -111,41 +109,83 @@ class UpdateCacheMiddleware(object):
             else:
                 key_prefix = self.key_prefix
             if self.post_process_response:
-                response = self.post_process_response(
-                    response,
-                    request
-                )
+                response = self.post_process_response(response, request)
 
             with RequestPath(request, self.only_get_keys, self.forget_get_keys):
                 cache_key = learn_cache_key(
-                    request,
-                    response,
-                    timeout,
-                    key_prefix
+                    request, response, timeout, key_prefix
                 )
 
                 if self.remember_all_urls:
                     self.remember_url(request, cache_key, timeout)
 
-            cache.set(cache_key, response, timeout)
+            self.cache.set(cache_key, response, timeout)
 
         if self.post_process_response_always:
-            response = self.post_process_response_always(
-                response,
-                request
-            )
+            response = self.post_process_response_always(response, request)
 
         return response
 
     def remember_url(self, request, cache_key, timeout):
+        """
+        Function to remember a newly cached URL.
+
+        All cached URLs are remembered in a dictionary
+        in the cache under REMEMBERED_URLS_KEY.
+
+        If USE_MEMCACHED_CAS is True, we try to use CAS (check and set)
+        to set the dictionary via self.cache._cache.cas to avoid missing
+        cached URLs in high traffic environments.cache._cache.cas.
+
+        See Issue #7 for more information:
+        https://github.com/peterbe/django-fancy-cache/issues/7
+        """
         url = request.get_full_path()
-        remembered_urls = cache.get(REMEMBERED_URLS_KEY, {})
+        if USE_MEMCACHED_CAS is True:
+            # Memcached check-and-set is available.
+            # Try using check-and-set to avoid a race condition
+            # in remembering urls; if this fails, fallback to cache.set.
+            result = self._remember_url_cas(url, cache_key)
+            if result:
+                # Remembered URLs have been successfully saved
+                # via Memcached CAS.
+                return
+
+        remembered_urls = self.cache.get(REMEMBERED_URLS_KEY, {})
         remembered_urls[url] = cache_key
-        cache.set(
-            REMEMBERED_URLS_KEY,
-            remembered_urls,
-            LONG_TIME
-        )
+        self.cache.set(REMEMBERED_URLS_KEY, remembered_urls, LONG_TIME)
+
+    def _remember_url_cas(self, url, cache_key):
+        """
+        Helper function to use Memcached CAS to store remembered URLs.
+        This addresses race conditions when using Memcached.
+        See Issue #7: https://github.com/peterbe/django-fancy-cache/issues/7.
+        """
+        result = False
+        tries = 0  # Make sure an unexpected error doesn't cause a loop
+        while result is False and tries <= 100:
+            remembered_urls, cas_token = self.cache._cache.gets(
+                REMEMBERED_URLS_KEY
+            )
+
+            if remembered_urls is None:
+                # No cache entry; set the cache using `cache.set`.
+                return False
+
+            remembered_urls[url] = cache_key
+
+            result = self.cache._cache.cas(
+                REMEMBERED_URLS_KEY, remembered_urls, cas_token, LONG_TIME
+            )
+
+            tries += 1
+
+        if result is False:
+            LOGGER.error(
+                "Django-fancy-cache failed to save using CAS after %s tries.",
+                tries,
+            )
+        return result
 
 
 class FetchFromCacheMiddleware(object):
@@ -156,6 +196,7 @@ class FetchFromCacheMiddleware(object):
     FetchFromCacheMiddleware must be the last piece of middleware in
     MIDDLEWARE_CLASSES so that it'll get called last during the request phase.
     """
+
     def process_request(self, request):
         """
         Checks whether the page is already cached and returns the cached
@@ -166,18 +207,18 @@ class FetchFromCacheMiddleware(object):
             # then we're nosy
             cache_key = request.get_full_path()
             if response is None:
-                cache_key += '__misses'
+                cache_key += "__misses"
             else:
-                cache_key += '__hits'
+                cache_key += "__hits"
             cache_key = md5(cache_key)
-            if cache.get(cache_key) is None:
-                cache.set(cache_key, 0, LONG_TIME)
-            cache.incr(cache_key)
+            if self.cache.get(cache_key) is None:
+                self.cache.set(cache_key, 0, LONG_TIME)
+            self.cache.incr(cache_key)
         return response
 
     def _process_request(self, request):
         if self.cache_anonymous_only:
-            if not hasattr(request, 'user'):
+            if not hasattr(request, "user"):
                 raise ImproperlyConfigured(
                     "The Django cache middleware with "
                     "CACHE_MIDDLEWARE_ANONYMOUS_ONLY=True requires "
@@ -187,16 +228,16 @@ class FetchFromCacheMiddleware(object):
                     "before the CacheMiddleware."
                 )
 
-        if not request.method in ('GET', 'HEAD'):
+        if request.method not in ("GET", "HEAD"):
             request._cache_update_cache = False
             # Don't bother checking the cache.
             return None
 
-        #if (
+        # if (
         #    request.GET and
         #    not callable(self.key_prefix) and
         #    not self.only_get_keys
-        #):
+        # ):
         #    request._cache_update_cache = False
         #    # Default behaviour for requests with GET parameters: don't bother
         #    # checking the cache.
@@ -225,7 +266,7 @@ class FetchFromCacheMiddleware(object):
             # No cache information available, need to rebuild.
             return None
 
-        response = cache.get(cache_key, None)
+        response = self.cache.get(cache_key, None)
         if response is None:
             request._cache_update_cache = True
             # No cache information available, need to rebuild.
@@ -234,8 +275,7 @@ class FetchFromCacheMiddleware(object):
         request._cache_update_cache = False
         if self.post_process_response_always:
             response = self.post_process_response_always(
-                response,
-                request=request
+                response, request=request
             )
 
         return response
@@ -289,41 +329,47 @@ class CacheMiddleware(UpdateCacheMiddleware, FetchFromCacheMiddleware):
         of the number of times a `cache_page` hits and misses.
 
     """
-    def __init__(self,
-                 cache_timeout=settings.CACHE_MIDDLEWARE_SECONDS,
-                 key_prefix=settings.CACHE_MIDDLEWARE_KEY_PREFIX,
-                 cache_anonymous_only=getattr(
-                     settings,
-                     'CACHE_MIDDLEWARE_ANONYMOUS_ONLY',
-                     False
-                 ),
-                 patch_headers=False,
-                 post_process_response=None,
-                 post_process_response_always=None,
-                 only_get_keys=None,
-                 forget_get_keys=None,
-                 remember_all_urls=getattr(
-                     settings,
-                     'FANCY_REMEMBER_ALL_URLS',
-                     False
-                 ),
-                 remember_stats_all_urls=getattr(
-                     settings,
-                     'FANCY_REMEMBER_STATS_ALL_URLS',
-                     False
-                 ),
-                 ):
+
+    def __init__(
+        self,
+        *args,
+        cache_timeout=settings.CACHE_MIDDLEWARE_SECONDS,
+        key_prefix=settings.CACHE_MIDDLEWARE_KEY_PREFIX,
+        cache_anonymous_only=getattr(
+            settings, "CACHE_MIDDLEWARE_ANONYMOUS_ONLY", False
+        ),
+        patch_headers=False,
+        post_process_response=None,
+        post_process_response_always=None,
+        only_get_keys=None,
+        forget_get_keys=None,
+        remember_all_urls=getattr(settings, "FANCY_REMEMBER_ALL_URLS", False),
+        remember_stats_all_urls=getattr(
+            settings, "FANCY_REMEMBER_STATS_ALL_URLS", False
+        ),
+        **kwargs
+    ):
         self.patch_headers = patch_headers
         self.cache_timeout = cache_timeout
         self.key_prefix = key_prefix
         self.cache_anonymous_only = cache_anonymous_only
         self.post_process_response = post_process_response
         self.post_process_response_always = post_process_response_always
-        if isinstance(only_get_keys, basestring):
+        if isinstance(only_get_keys, str):
             only_get_keys = [only_get_keys]
         self.only_get_keys = only_get_keys
-        if isinstance(forget_get_keys, basestring):
+        if isinstance(forget_get_keys, str):
             forget_get_keys = [forget_get_keys]
         self.forget_get_keys = forget_get_keys
         self.remember_all_urls = remember_all_urls
         self.remember_stats_all_urls = remember_stats_all_urls
+
+        try:
+            cache_alias = kwargs["cache_alias"]
+            if cache_alias is None:
+                cache_alias = DEFAULT_CACHE_ALIAS
+        except KeyError:
+            cache_alias = settings.CACHE_MIDDLEWARE_ALIAS
+
+        self.cache_alias = cache_alias
+        self.cache = caches[self.cache_alias]
